package plugins

import (
	"fmt"
	"sync"

	"github.com/christalib/structs"
	"github.com/ns3777k/go-shodan/v4/shodan"
	"github.com/sirupsen/logrus"
	"github.com/spf13/viper"
	"gitlab.dcso.lolcat/LABS/styx/filters"
	"gitlab.dcso.lolcat/LABS/styx/models"
)

// ShodanPlugin defines the general ShodanPlugin structure.
type ShodanPlugin struct {
	Client      *shodan.Client
	ShodanChan  chan *shodan.HostData
	StopChan    chan bool
	StoppedChan chan bool
	Running     bool
}

// Initialize initialises the certstream configuration.
func (s *ShodanPlugin) Initialize() bool {
	if !viper.GetBool("shodan.activated") {
		return false
	}
	logrus.Info("shodan plugin is activated")
	return true
}

// Run runs the Shodan plugin.
func (s *ShodanPlugin) Run(wg *sync.WaitGroup) {
	if !s.Running {
		s.StoppedChan = make(chan bool)
		wg.Add(1)
		go s.doRun()
		s.Running = true
	}
}

// Stop stops the Shodan plugin.
func (s *ShodanPlugin) Stop(wg *sync.WaitGroup) {
	if s.Running {
		s.StoppedChan = make(chan bool)
		close(s.StopChan)
		<-s.StopChan
		wg.Done()
		s.Running = false
	}
}

func (s *ShodanPlugin) doRun() {
	for {
		select {
		default:
			banner, ok := <-s.ShodanChan
			if !ok {
				logrus.Error("channel is closed")
				break
			}

			shodanNode := models.BuildShodanNode(banner)
			// first filter poc
			if shodanNode.Data.HTML != "" {
				if !filters.RunIPFilters(shodanNode.Data.IP) {
					hostnames := shodanNode.Data.Hostnames
					var hostNotInFilters, domainNotInFilters bool
					if len(hostnames) != 0 {
						for _, hostname := range hostnames {
							hostNotInFilters = filters.RunDomainFilters(hostname)
							if hostNotInFilters {
								// saveSingleValues(conn, "shodan_stream", "hostname", shodanNode.ID, hostname)
							}
						}
					}
					domains := shodanNode.Data.Domains
					if len(domains) != 0 {
						for _, domain := range domains {
							domainNotInFilters = filters.RunDomainFilters(domain)
							// saveSingleValues(conn, "shodan_stream", "domain", shodanNode.ID, domain)
						}
					}
					if domainNotInFilters && hostNotInFilters {
						models.SaveShodanNode("raw_shodan.json", shodanNode)
						node := models.BuildNode("shodan", "shodan_stream", shodanNode.ID)
						models.SaveNode("nodes.json", node)
						edge := models.BuildEdge("shodan", structs.Map(shodanNode), structs.Map(node))
						models.SaveEdge(edge)
					}
				} else {
					fmt.Println("is akamai", shodanNode.Data.IP)
				}
			}
		}
	}
}
